@@ -18,11 +18,13 @@ from scipy.stats import rankdata
 
 from pandas.core.frame import DataFrame
 
+from sklearn import __version__ as sklearn_version
 from sklearn.model_selection import ParameterSampler
 from sklearn.model_selection._validation import _fit_and_score
 from sklearn.base import clone
 from joblib import Parallel, delayed
 
+from dstk.utils import list_of_dict_2_dict_of_list as l2d
 from ._base import BaseEnvironnement
 
 PyTorchEstimators = NewType('PyTorchEstimators', BaseEnvironnement)
@@ -102,22 +104,19 @@ class RandomizedSearchOnline:
     méthodes fit et fit_online, retourne une erreur.
     """
     def __init__(self,
-                 estimator: PyTorchEstimators,
-                 param_distributions: Dict[str, Any],
-                 n_iter: int=10,
-                 scoring: Optional[Union[str,
-                                         Callable,
-                                         List[str],
-                                         Tuple[str],
-                                         Dict[str, Callable]]]=None,
-                 n_jobs: Optional[int]=None,
-                 refit: Union[bool, str, Callable]=True,
-                 monte_carlo: int=1,
-                 verbose: int=0,
-                 pre_dispatch: Optional[Union[str, int]]='2*n_jobs',
-                 random_state: Optional[int]=None,
-                 error_score: str='raise',
-                 return_train_score: bool=False):
+        estimator: PyTorchEstimators,
+        param_distributions: Dict[str, Any],
+        n_iter: int=10,
+        scoring: Optional[Union[str, Callable, List[str], Tuple[str], Dict[str, Callable]]]=None,
+        n_jobs: Optional[int]=None,
+        refit: Union[bool, str, Callable]=True,
+        monte_carlo: int=1,
+        verbose: int=0,
+        pre_dispatch: Optional[Union[str, int]]='2*n_jobs',
+        random_state: Optional[int]=None,
+        error_score: str='raise',
+        return_train_score: bool=False
+    ):
         self.estimator = estimator
         self.param_distributions = param_distributions
         self.n_iter = n_iter
@@ -148,13 +147,15 @@ class RandomizedSearchOnline:
                 "ou égale à 1."
             )
 
-    def fit(self,
-            X: Iterable,
-            y: Optional[Iterable]=None,
-            idx_stop_train: Optional[int]=None,
-            gap: int=0,
-            idx_skip: Optional[Union[np.ndarray, List[int]]]=None,
-            **fit_params):
+    def fit(
+        self,
+        X: Iterable,
+        y: Optional[Iterable]=None,
+        idx_stop_train: Optional[int]=None,
+        gap: int=0,
+        idx_skip: Optional[Union[np.ndarray, List[int]]]=None,
+        **fit_params
+    ):
         r"""
         Applique le fit de l'estimateur sur l'univers des hyper-paramètres.
         
@@ -197,7 +198,7 @@ class RandomizedSearchOnline:
             )
 
         fit_params_scorers = fit_params.copy()
-        fit_params_scorers['rebuild'] = False
+        fit_params_scorers['build'] = False
         if callable(self.scoring):
             self.multimetric_ = False
             scorers = {
@@ -361,10 +362,12 @@ class RandomizedSearchOnline:
 
         return self
 
-    def _format_results(self,
-                        candidate_params: List[Dict[str, Any]],
-                        scorers: Union[Callable, Dict[str, Callable]],
-                        out: List[float]):
+    def _format_results(
+        self,
+        candidate_params: List[Dict[str, Any]],
+        scorers: Union[Callable, Dict[str, Callable]],
+        out: List[float]
+    ):
         r"""\
         Fontion de formatage des résultats interne à la classe.
         
@@ -393,12 +396,22 @@ class RandomizedSearchOnline:
         peut trouver dans Scikit-Learn à quelques adaptations près.
         """
         n_candidates = len(candidate_params)
-        if self.return_train_score:
-            (train_score_dicts, test_score_dicts, test_sample_counts, 
-             fit_time, score_time) = zip(*out)
+        if sklearn_version < '0.20':
+            if self.return_train_score:
+                (train_score_dicts, test_score_dicts,
+                 test_sample_counts, fit_time, score_time) = zip(*out)
+            else:
+                (test_score_dicts, test_sample_counts, fit_time,
+                 score_time) = zip(*out)
         else:
-            (test_score_dicts, test_sample_counts, fit_time,
-             score_time) = zip(*out)
+            out_ = l2d(out)
+            fit_failed = out_['fit_failed']
+            test_score_dicts = out_['test_scores']
+            # test_sample_counts = out_['n_test_samples']
+            fit_time = out_['fit_time']
+            score_time = out_['score_time']
+            if self.return_train_score:
+                train_score_dicts = out_['train_scores']
 
         test_scores = {
             key: np.asarray(
@@ -424,10 +437,14 @@ class RandomizedSearchOnline:
                 if monte_carlo:
                     for mc_i in range(self.monte_carlo):
                         results[f"mc{mc_i}_{key_name}"] = array[:, mc_i]
-                array_means = np.average(array, axis=1)
-                results[f"mean_{key_name}"] = array_means
-                array_stds = np.std(array, axis=1)
-                results[f"std_{key_name}"] = array_stds
+                if key_name != 'fit_failed':
+                    array_means = np.average(array, axis=1)
+                    results[f"mean_{key_name}"] = array_means
+                    array_stds = np.std(array, axis=1)
+                    results[f"std_{key_name}"] = array_stds
+                else:
+                    array_means = np.average(array, axis=1)
+                    results[f"ratio_{key_name}"] = array_means
                 if rank:
                     results[f"rank_{key_name}"] = np.asarray(
                         rankdata(-array_means, method='min'),
@@ -447,6 +464,8 @@ class RandomizedSearchOnline:
                     f"ou égal à 1 et non égal à {self.monte_carlo}."
                 )
 
+        if sklearn_version >= '0.20':
+            _store('fit_failed', fit_failed)
         _store('fit_time', fit_time)
         _store('score_time', score_time)
 
